import type { Equatable } from "@siteimprove/alfa-equatable";
import type { Serializable } from "@siteimprove/alfa-json";
import type * as json from "@siteimprove/alfa-json";
import type { Parser as CSSParser } from "./parser.js";
import { Token } from "./token.js";
/**
 * {@link https://drafts.csswg.org/css-syntax/#declaration}
 *
 * @public
 */
export declare class Declaration implements Iterable<Token>, Equatable, Serializable {
    static of(name: string, value: Iterable<Token>, important?: boolean): Declaration;
    private readonly _name;
    private readonly _value;
    private readonly _important;
    private constructor();
    get name(): string;
    get value(): Array<Token>;
    get important(): boolean;
    [Symbol.iterator](): Iterator<Token>;
    equals(value: unknown): value is this;
    toJSON(): Declaration.JSON;
    toString(): string;
}
/**
 * @public
 */
export declare namespace Declaration {
    interface JSON {
        [key: string]: json.JSON;
        name: string;
        value: Array<Token.JSON>;
        important: boolean;
    }
    /**
     * {@link https://drafts.csswg.org/css-syntax/#consume-a-declaration}
     */
    const consume: CSSParser<Declaration>;
    /**
     * {@link https://drafts.csswg.org/css-syntax/#parse-a-declaration}
     */
    const parse: CSSParser<Declaration>;
    /**
     * {@link https://drafts.csswg.org/css-syntax/#consume-a-list-of-declarations}
     */
    const consumeList: CSSParser<Iterable<Declaration>>;
    /**
     * {@link https://drafts.csswg.org/css-syntax/#parse-a-list-of-declarations}
     */
    const parseList: CSSParser<Iterable<Declaration>>;
}
//# sourceMappingURL=declaration.d.ts.map