import type { Hash } from "@siteimprove/alfa-hash";
import { Option } from "@siteimprove/alfa-option";
import { Parser } from "@siteimprove/alfa-parser";
import { type Parser as CSSParser, Token } from "../../syntax/index.js";
import type { Keyword } from "../keyword.js";
import { LengthPercentage } from "../numeric/index.js";
import type { PartiallyResolvable, Resolvable } from "../resolvable.js";
import { Value } from "../value.js";
import { Keywords } from "./keywords.js";
/**
 * @public
 */
export declare class Side<S extends Keywords.Vertical | Keywords.Horizontal = Keywords.Vertical | Keywords.Horizontal, O extends LengthPercentage = LengthPercentage> extends Value<"side", Value.HasCalculation<[O]>> implements Resolvable<Side.Canonical<S>, Side.Resolver>, PartiallyResolvable<Side.PartiallyResolved<S>, Side.PartialResolver> {
    static of<S extends Keywords.Vertical | Keywords.Horizontal>(side: S): Side<S, never>;
    static of<S extends Keywords.Vertical | Keywords.Horizontal, O extends LengthPercentage>(side: S, offset: O): Side<S, O>;
    static of<S extends Keywords.Vertical | Keywords.Horizontal, O extends LengthPercentage>(side: S, offset: Option<O>): Side<S, O>;
    private readonly _side;
    private readonly _offset;
    private constructor();
    get side(): S;
    get offset(): Option<O>;
    resolve(resolver: Side.Resolver): Side.Canonical<S>;
    partiallyResolve(resolver: Side.PartialResolver): Side.PartiallyResolved<S>;
    equals(value: unknown): value is this;
    hash(hash: Hash): void;
    toJSON(): Side.JSON;
    toString(): string;
}
/**
 * @public
 */
export declare namespace Side {
    type Canonical<S extends Keywords.Vertical | Keywords.Horizontal> = Side<S, LengthPercentage.Canonical>;
    type PartiallyResolved<S extends Keywords.Vertical | Keywords.Horizontal> = Side<S, LengthPercentage.PartiallyResolved>;
    interface JSON extends Value.JSON<"side"> {
        side: Keyword.JSON;
        offset: LengthPercentage.JSON | null;
    }
    type Resolver = LengthPercentage.Resolver;
    type PartialResolver = LengthPercentage.PartialResolver;
    function isSide(value: unknown): value is Side;
    const parseHorizontalKeywordValue: CSSParser<Side<Keyword.ToKeywords<"right" | "left">, LengthPercentage>>;
    const parseHorizontalKeyword: CSSParser<Keyword<"center"> | Side<Keyword.ToKeywords<"right" | "left">, never>>;
    const parseHorizontal: Parser<import("@siteimprove/alfa-slice").Slice<Token>, Keyword<"center"> | Side<Keyword.ToKeywords<"right" | "left">, LengthPercentage> | Side<Keyword.ToKeywords<"right" | "left">, never>, string, []>;
    const parseVerticalKeywordValue: CSSParser<Side<Keyword.ToKeywords<"top" | "bottom">, LengthPercentage>>;
    const parseVerticalKeyword: CSSParser<Keyword<"center"> | Side<Keyword.ToKeywords<"top" | "bottom">, never>>;
    const parseVertical: Parser<import("@siteimprove/alfa-slice").Slice<Token>, Keyword<"center"> | Side<Keyword.ToKeywords<"top" | "bottom">, LengthPercentage> | Side<Keyword.ToKeywords<"top" | "bottom">, never>, string, []>;
}
//# sourceMappingURL=side.d.ts.map