import type { Hash } from "@siteimprove/alfa-hash";
import type { Parser as CSSParser } from "../../syntax/index.js";
import { Keyword } from "../keyword.js";
import { LengthPercentage } from "../numeric/index.js";
import type { PartiallyResolvable, Resolvable } from "../resolvable.js";
import { Value } from "../value.js";
import { BasicShape } from "./basic-shape.js";
/**
 * {@link https://drafts.csswg.org/css-shapes/#typedef-shape-radius}
 *
 * @public
 */
export declare class Radius<R extends LengthPercentage | Radius.Side = LengthPercentage | Radius.Side> extends BasicShape<"radius", Value.HasCalculation<[R]>> implements Resolvable<Radius.Canonical, Radius.Resolver>, PartiallyResolvable<Radius.PartiallyResolved, Radius.PartialResolver> {
    static of<R extends LengthPercentage | Radius.Side>(value: R): Radius<R>;
    private readonly _value;
    private constructor();
    get value(): R;
    resolve(resolver: Radius.Resolver): Radius.Canonical;
    partiallyResolve(resolver: Radius.PartialResolver): Radius.PartiallyResolved;
    equals(value: Radius): boolean;
    equals(value: unknown): value is this;
    hash(hash: Hash): void;
    toJSON(): Radius.JSON;
    toString(): string;
}
/**
 * @public
 */
export declare namespace Radius {
    type Canonical = Radius<LengthPercentage.Canonical | Side>;
    type PartiallyResolved = Radius<LengthPercentage.PartiallyResolved | Side>;
    interface JSON extends BasicShape.JSON<"radius"> {
        value: LengthPercentage.JSON | Keyword.JSON;
    }
    type Resolver = LengthPercentage.Resolver;
    type PartialResolver = LengthPercentage.PartialResolver;
    type Side = Side.Closest | Side.Farthest;
    namespace Side {
        /**
         * {@link https://drafts.csswg.org/css-shapes/#closest-side}
         */
        type Closest = Keyword<"closest-side">;
        /**
         * {@link https://drafts.csswg.org/css-shapes/#farthest-side}
         */
        type Farthest = Keyword<"farthest-side">;
    }
    function isRadius(value: unknown): value is Radius;
    const parse: CSSParser<Radius>;
}
//# sourceMappingURL=radius.d.ts.map